<?php

/***********************************************************
 * Name:     CustomSidebar
 * Desc:     Easy system for specifying custom sidebars on a per-page basis
 *
 * Version:  0.1.0
 *
 * Author:   Swiftly Tilting (contact@swiftlytilting.com)
 * Homepage: http://www.mediawiki.org/wiki/Extension:CustomSidebar
 *           http://www.swiftlytilting.com/
 *
 * License:  Free for any use
 *
 ***********************************************************
 */
 
$wgExtensionCredits['parserhook'][] = array(
       'name' => 'CustomSidebar',
       'author' =>'SwiftlyTilting', 
       'url' => 'http://www.mediawiki.org/wiki/Extension:CustomSidebar', 
       'description' => 'Easy system for specifying custom sidebars on a per-page basis',
       'descriptionmsg' => "customsidebar-desc", // Same as above but name of a message, for i18n - string, added in 1.12.0
		 'version' => '0.1.0',
		 'path' => __FILE__,
       );


$wgNewSideBar = false;

//Avoid unstubbing $wgParser on setHook() too early on modern (1.12+) MW versions, as per r35980
if ( defined( 'MW_SUPPORTS_PARSERFIRSTCALLINIT' ) ) {
	$wgHooks['ParserFirstCallInit'][] = 'efCustomSideBarInit';
} else { // Otherwise do things the old fashioned way
	$wgExtensionFunctions[] = 'efSampleParserInit';
}
 
function efCustomSideBarInit() {
	global $wgParser;
	$wgParser->setHook( 'sidebar', 'efCustomSideBar' );
	return true;
}
 
function efCustomSideBar( $input, $args, $parser ) {
	global $wgNewSideBar;
	$wgNewSideBar = $input;

}

$wgHooks['SkinBuildSidebar'][] = 'fnSidebarHook';

function fnSidebarHook($skin, &$bar) 
{
	   // this is mostly just the standard sidebar processing function with a custom loader
	
		global $parserMemc, $wgEnableSidebarCache, $wgSidebarCacheExpiry;
		global $wgLang, $wgNewSideBar, $wgTitle;
		
		// custom loader
		if ($wgNewSideBar)
		{	if (strpos(trim($wgNewSideBar), '*') === 0)
			{	$text = $wgNewSideBar;
			}
			else
			{	$titleFromText = $wgTitle->newFromText($wgNewSideBar);

				$article = new Article($titleFromText);
 
				$text = $article->getContent();
			}

			$lines = explode( "\n",  $text );
		}
		else
		{  return true;
		}
		
		
		wfProfileIn( __METHOD__ );

		$key = wfMemcKey( 'sidebar', $wgLang->getCode() );

		if ( $wgEnableSidebarCache ) {
			$cachedsidebar = $parserMemc->get( $key );
			if ( $cachedsidebar ) {
				wfProfileOut( __METHOD__ );
				return $cachedsidebar;
			}
		}

		$bar = array();

		$heading = '';
		foreach ($lines as $line) {
			if (strpos($line, '*') !== 0)
				continue;
			if (strpos($line, '**') !== 0) {
				$line = trim($line, '* ');
				$heading = $line;
				if( !array_key_exists($heading, $bar) ) $bar[$heading] = array();
			} else {
				if (strpos($line, '|') !== false) { // sanity check
					$line = array_map('trim', explode( '|' , trim($line, '* '), 2 ) );
					$link = wfMsgForContent( $line[0] );
					if ($link == '-')
						continue;

					$text = wfMsgExt($line[1], 'parsemag');
					if (wfEmptyMsg($line[1], $text))
						$text = $line[1];
					if (wfEmptyMsg($line[0], $link))
						$link = $line[0];

					if ( preg_match( '/^(?:' . wfUrlProtocols() . ')/', $link ) ) {
						$href = $link;
					} else {
						$title = Title::newFromText( $link );
						if ( $title ) {
							$title = $title->fixSpecialName();
							$href = $title->getLocalURL();
						} else {
							$href = 'INVALID-TITLE';
						}
					}

					$bar[$heading][] = array(
						'text' => $text,
						'href' => $href,
						'id' => 'n-' . strtr($line[1], ' ', '-'),
						'active' => false
					);
				} else { continue; }
			}
		}
		if ( $wgEnableSidebarCache ) $parserMemc->set( $key, $bar, $wgSidebarCacheExpiry );
		wfProfileOut( __METHOD__ );
		
		return true;	

}

?>